#!/bin/bash

###
### Установка в WEB-Сервер-КС ГОСТ-сертификата
###
### Copyright (c) 2024, ООО Кейсистемс
###
### Версия 20240423
###

deb_versions=(9 11 12) #поддерживаемые версии контейнеров

# Цвета
ErrColor='\033[1;31m'
SuccColor='\033[0;32m'
WarnColor='\033[0;33m'
QuestColor='\033[0;34m'
NoColor='\033[0m'
sOK="[===${SuccColor}OK${NoColor}===]="
wOK="[==${WarnColor}Инфо${NoColor}==]="
rOK="[${QuestColor}==>${NoColor}]"
eRR="[=${ErrColor}Ошибка${NoColor}=]="
emptE="   "

# проверка наличия элемента в массиве
contains () {
    local list=$1[@]
    local elem=$2
    local i
    for i in "${!list}"
    do
     if [ "$i" == "${elem}" ] ; then
         return 0
     fi
    done
    return 1
}

dks_bashfile_dir="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
while [[ "$#" -gt 0 ]]
  do
    case $1 in
      --contname)
        cont_path="$2"
        ;;

      --certname)
        cert_name="$2"
        ;;

      --help)
        show_help="1"
        ;;
    esac
    shift
done

if [ -z "${cert_name}" ] ; then
  cert_name="gost"
fi
if [ -z "${cont_path}" ] ; then
  show_help="1"
else
  cont_path="/opt/${cont_path}"
fi

if [ ! -z "${show_help}" ]; then
echo -e "
Пример вызова: bash install.sh --contname wsks-6.0.9_443
 параметры         по умолчанию   описание
 ---------------   ------------   ------------------------------------------------------------------------
  --contname                      имя web-контейнера, узнать можно командой: docker ps или podman ps
  --certname          gost        имя файла *.key *.crt. В архиве по умолчанию самоподписанный сертификат.
  --help                          вызвать справочник
"
exit 1
fi

#  Проверяем наличие сертификата
if [[ ! -f "${dks_bashfile_dir}/certs/${cert_name}.crt" || ! -f "${dks_bashfile_dir}/certs/${cert_name}.key" ]]; then
  echo -e "${eRR}Не найден сертификат в ${dks_bashfile_dir}/certs !"
  exit 1
fi

#  Проверяем наличие каталога с контейнером
dks_inst_path=$($cont_path/ctl/ws_const_var.sh dks_inst_path)
if [ $? -ne 0 ]; then
  echo -e "${eRR}Исправьте расположение контейнера!"
  exit 1
fi

# определяем тип и порт web-а
ws_type=$($cont_path/ctl/ws_const_var.sh       ws_type)
ws_port=$($cont_path/ctl/ws_const_var.sh       ws_port)
if [[ "${ws_type}" != "nginx" && "${ws_type}" != "angie" && "${ws_type}" != "apache2" ]]; then
    echo -e "${eRR}На данный момент поддерживается только контейнер nginx, angie и apache2!"
    exit 1
fi

# Проверяем существование контейнера в docker/podman
dks_cont_name=$($cont_path/ctl/ws_const_var.sh dks_cont_name)
contsoft=$($cont_path/ctl/ws_const_var.sh      contsoft)
dks_exists=$(${contsoft} container ls -a | grep ${dks_cont_name})
if [ -z "$dks_exists" ]; then
  echo -e "${eRR}Проверка существования контейнера ${dks_cont_name}..."
  echo -e "${emptE}Контейнер ${dks_cont_name} не существует."
  echo ""
  exit 1
fi
# Определяем запущен ли контейнер
dks_status=$(${contsoft} inspect -f '{{.State.Running}}' ${dks_cont_name})
if [ "${dks_status}" = "false" ]; then
  echo -e "${eRR}Проверка запуска контейнера ${dks_cont_name}..."
  echo -e "${emptE}Контейнер ${dks_cont_name} не запущен."
  echo ""
  exit 1
fi

# определяем версию контейнера
deb_id="$(echo $(${contsoft} exec ${dks_cont_name} bash -c 'cat /etc/os-release | grep ID=debian') | awk -F '=' '{print $2}')"
if [ "${deb_id}" != "debian" ]; then
   echo -e "${eRR}Версия контейнера не debian!"
   exit 1
fi
deb_version="$(echo $(${contsoft} exec ${dks_cont_name} bash -c 'cat /etc/debian_version') | awk -F '.' '{print $1}')"
echo -e "${wOK}Версия контейнера: ${deb_id} ${deb_version}"

if ! contains deb_versions "${deb_version}"; then
    echo -e "${eRR}Контейнер не поддерживается!"
    exit 1
fi

is_deb_installed="$(${contsoft} exec ${dks_cont_name} bash -c 'dpkg -s libengine-gost-openssl1.1 | grep "Status"')"
if [ -n "$is_deb_installed" ]
then
   echo -e "${wOK}Идет установка пакета libengine-gost-openssl1.1 в контейнер..."
   echo -e "${sOK}Пакет libengine-gost-openssl1.1 в контейнере уже установлен."
else
   echo -e "${wOK}Идет установка пакета libengine-gost-openssl1.1 в контейнер..."
   ${contsoft} exec ${dks_cont_name} bash -c "mkdir -p /opt/libengine-gost"
   ${contsoft} cp ${dks_bashfile_dir}/deb/${deb_version} ${dks_cont_name}:/opt/libengine-gost
   ${contsoft} exec ${dks_cont_name} bash -c "dpkg -i /opt/libengine-gost/${deb_version}/*.deb"
   is_deb_installed="$(${contsoft} exec ${dks_cont_name} bash -c 'dpkg -s libengine-gost-openssl1.1 | grep "Status"')"
   if [ -n "$is_deb_installed" ]
   then
      echo -e "${sOK}Пакет libengine-gost-openssl1.1 в контейнере установлен успешно."
   else
      echo -e "${eRR}Ошибка установки пакета libengine-gost-openssl1.1 в контейнер!"
      exit 1
   fi
fi

echo -e "${wOK}Идет конфигурирование /etc/ssl/openssl.cnf в контейнере..."
openssl_cnf='openssl.cnf'
${contsoft} cp ${dks_cont_name}:/etc/ssl/openssl.cnf ${dks_bashfile_dir}/${openssl_cnf}
bash ${dks_bashfile_dir}/src/ini.sh set ${dks_bashfile_dir}/${openssl_cnf} ""             openssl_conf       openssl_def
bash ${dks_bashfile_dir}/src/ini.sh set ${dks_bashfile_dir}/${openssl_cnf} openssl_def    engines            engine_section
bash ${dks_bashfile_dir}/src/ini.sh set ${dks_bashfile_dir}/${openssl_cnf} engine_section gost               gost_section
bash ${dks_bashfile_dir}/src/ini.sh set ${dks_bashfile_dir}/${openssl_cnf} gost_section   engine_id          gost
bash ${dks_bashfile_dir}/src/ini.sh set ${dks_bashfile_dir}/${openssl_cnf} gost_section   default_algorithms ALL
bash ${dks_bashfile_dir}/src/ini.sh set ${dks_bashfile_dir}/${openssl_cnf} gost_section   CRYPT_PARAMS       id-Gost28147-89-CryptoPro-A-ParamSet
${contsoft} cp ${dks_bashfile_dir}/${openssl_cnf} ${dks_cont_name}:/etc/ssl/openssl.cnf
rm -f ${dks_bashfile_dir}/${openssl_cnf}

echo -e "${wOK}Идет конфигурирование ${ws_type}..."
cp ${dks_bashfile_dir}/certs/${cert_name}.crt ${cont_path}/ssl/${cert_name}.crt
cp ${dks_bashfile_dir}/certs/${cert_name}.key ${cont_path}/ssl/${cert_name}.key

# конфигурирование nginx или angie
if [[ "${ws_type}" = "nginx" || "${ws_type}" = "angie" ]]; then
    mv ${cont_path}/conf.ks/sslks.conf ${cont_path}/conf.ks/sslks.conf.bak
cat > ${cont_path}/conf.ks/sslks.conf << EOF
    ssl_certificate     /etc/${ws_type}/ssl/${cert_name}.crt;
    ssl_certificate_key /etc/${ws_type}/ssl/${cert_name}.key;

    ### только при наличии издателя
    # ssl_client_certificate /etc/${ws_type}/ssl/ca.crt;

    ssl_protocols TLSv1.2;
    ssl_prefer_server_ciphers on;

    ssl_ciphers GOST2012-GOST8912-GOST8912:GOST2001-GOST89-GOST89:HIGH:MEDIUM;
    ssl_ecdh_curve              secp384r1;
    ssl_session_cache           shared:SSL:10m;
    ssl_session_tickets         off;
    ssl_stapling                on;
    ssl_stapling_verify         on;
EOF
fi

# конфигурирование apache2
if [[ "${ws_type}" = "apache2" ]]; then
    mv ${cont_path}/conf.ks/sslks.conf ${cont_path}/conf.ks/sslks.conf.bak
    mv ${cont_path}/conf.ks/vhost_ks.conf ${cont_path}/conf.ks/vhost_ks.conf.bak

cat > ${cont_path}/conf.ks/sslks.conf << EOF
  # Убираем компрессию для предотвращения CRIME атак
  SSLCompression          off

  SSLProtocol             all -SSLv3 -SSLv2 -TLSv1.3
  # all соответствует разрешенным +SSLv2 +SSLv3 +TLSv1 +TLSv1.1 +TLSv1.2 +TLSv1.3
  # -SSLv3 -SSLv2 -TLSv1.3 -> исключает использование этих протоколов из набора all
  # ГОСТ не работает с TLSv1.3

  # Использование SSL-кодов только с высокой степенью защиты
  SSLCipherSuite HIGH:!aNULL:!MD5:!3DES

  #Набор шифров
  # SSLCipherSuite EECDH+AESGCM:EDH+AESGCM:AES256+EECDH:AES256+EDH
  SSLCipherSuite GOST2012-GOST8912-GOST8912:GOST2001-GOST89-GOST89:HIGH:MEDIUM

  # Использоваться предпочтения кодов сервера, а не клиентские предпочтения.
  SSLHonorCipherOrder     on

  SSLSessionTickets       off

  SSLUseStapling Off
  SSLStaplingCache "shmcb:logs/ssl_stapling(32768)"

  # пишут про циклический кэш, который увеличивает быстродействие
  SSLSessionCache "shmcb:/logs/ssl_gcache_data(512000)"
  SSLSessionCacheTimeout 600

  # Опция, если нужно пускать только тех, у кого есть клиентский сертификат (2-хсторонний SSL)
  # SSLVerifyClient require
EOF

cat > ${cont_path}/conf.ks/vhost_ks.conf << EOF
<VirtualHost *:${ws_port}>

  RequestHeader set "X-Forwarded-Proto" expr=%{REQUEST_SCHEME}
  ProxyPreserveHost On

  RewriteEngine on

  SSLEngine on
  SSLCertificateKeyFile /etc/apache2/ssl/gost.key
  SSLCertificateFile    /etc/apache2/ssl/gost.crt

  # enable HTTP/2, if available
  Protocols h2 http/1.1

  # HTTP Strict Transport Security (mod_headers is required) (63072000 seconds)
  Header always set Strict-Transport-Security "max-age=63072000"

  IncludeOptional conf.ks/vhost.ks/*.conf

  #CustomLog ${APACHE_LOG_DIR}/other_vhosts_access.log vhost_combined

</VirtualHost>
EOF
fi

echo -e "${wOK}Идет перезапуск контейнера ${dks_cont_name}..."
# ${contsoft} exec -ti ${dks_cont_name} bash -c "/etc/init.d/${ws_type} reload"
systemctl restart ${dks_cont_name}
